@@ -12,7 +12,7 @@ use strict;
 use vars qw($VERSION);
 use Image::ExifTool qw(:DataAccess :Utils);
 
-$VERSION = '1.14';
+$VERSION = '1.15';
 
 sub MDItemLocalTime($);
 sub ProcessATTR($$$);
@@ -364,6 +364,23 @@ sub MDItemLocalTime($)
     return $val;
 }
 
+#------------------------------------------------------------------------------
+# Call system command, redirecting all I/O to /dev/null
+# Inputs: system arguments
+# Returns: system return code
+sub System
+{
+    my ($oldout, $olderr);
+    open($oldout, ">&STDOUT");
+    open($olderr, ">&STDERR");
+    open(STDOUT, '>', '/dev/null');
+    open(STDERR, '>', '/dev/null');
+    my $result = system(@_);
+    open(STDOUT, ">&", $oldout);
+    open(STDERR, ">&", $olderr);
+    return $result;
+}
+
 #------------------------------------------------------------------------------
 # Set MacOS MDItem and XAttr tags from new tag values
 # Inputs: 0) ExifTool ref, 1) file name, 2) list of tags to set
@@ -376,7 +393,7 @@ sub SetMacOSTags($$$)
     my $tag;
 
     foreach $tag (@$setTags) {
-        my ($nvHash, $f, $v, $attr, $cmd, $err, $silentErr);
+        my ($nvHash, $attr, @cmd, $err, $silentErr);
         my $val = $et->GetNewValue($tag, \$nvHash);
         next unless $nvHash;
         my $overwrite = $et->IsOverwriting($nvHash);
@@ -389,7 +406,6 @@ sub SetMacOSTags($$$)
             }
         }
         if ($tag eq 'MDItemFSCreationDate' or $tag eq 'FileCreateDate') {
-            ($f = $file) =~ s/'/'\\''/g;
             # convert to local time if value has a time zone
             if ($val =~ /[-+Z]/) {
                 my $time = Image::ExifTool::GetUnixTime($val, 1);
@@ -397,17 +413,15 @@ sub SetMacOSTags($$$)
                 $val =~ s/[-+].*//; # remove time zone
             }
             $val =~ s{(\d{4}):(\d{2}):(\d{2})}{$2/$3/$1};   # reformat for setfile
-            $cmd = "/usr/bin/setfile -d '${val}' '${f}'";
+            push @cmd, '/usr/bin/setfile', '-d', $val, $file;
         } elsif ($tag eq 'MDItemUserTags') {
             # (tested with "tag" version 0.9.0)
-            ($f = $file) =~ s/'/'\\''/g;
             my @vals = $et->GetNewValue($nvHash);
             if ($overwrite < 0 and @{$$nvHash{DelValue}}) {
                 # delete specified tags
                 my @dels = @{$$nvHash{DelValue}};
-                s/'/'\\''/g foreach @dels;
                 my $del = join ',', @dels;
-                $err = system "/usr/local/bin/tag -r '${del}' '${f}'>/dev/null 2>&1";
+                $err = System('/usr/local/bin/tag', '-r', $del, $file);
                 unless ($err) {
                     $et->VerboseValue("- $tag", $del);
                     $result = 1;
@@ -416,43 +430,39 @@ sub SetMacOSTags($$$)
             }
             unless (defined $err) {
                 # add new tags, or overwrite or delete existing tags
-                s/'/'\\''/g foreach @vals;
                 my $opt = $overwrite > 0 ? '-s' : '-a';
                 $val = @vals ? join(',', @vals) : '';
-                $cmd = "/usr/local/bin/tag $opt '${val}' '${f}'";
+                push @cmd, '/usr/local/bin/tag', $opt, $val, $file;
                 $et->VPrint(1,"    - $tag = (all)\n") if $overwrite > 0;
                 undef $val if $val eq '';
             }
         } elsif ($delXAttr{$tag}) {
-            ($f = $file) =~ s/'/'\\''/g;
-            $cmd = "/usr/bin/xattr -d $delXAttr{$tag} '${f}'";
+            push @cmd, '/usr/bin/xattr', '-d', $delXAttr{$tag}, $file;
             $silentErr = 256;   # (will get this error if attribute doesn't exist)
         } else {
-            ($f = $file) =~ s/(["\\])/\\$1/g;   # escape necessary characters for script
-            $f =~ s/'/'"'"'/g;
+            my ($f, $v);
+            ($f = $file) =~ s/([\\"])/\\$1/g;   # escape backslashes and quotes for AppleScript
             if ($tag eq 'MDItemFinderComment') {
                 # (write finder comment using osascript instead of xattr
                 # because it is more work to construct the necessary bplist)
                 $val = '' unless defined $val;  # set to empty string instead of deleting
                 $v = $et->Encode($val, 'UTF8');
-                $v =~ s/(["\\])/\\$1/g;
-                $v =~ s/'/'"'"'/g;
+                $v =~ s/([\\"])/\\$1/g;
                 $attr = 'comment';
             } else { # $tag eq 'MDItemFSLabel'
                 $v = $val ? 8 - $val : 0;       # convert from label to label index (0 for no label)
                 $attr = 'label index';
             }
-            $cmd = qq(/usr/bin/osascript -e 'set fp to POSIX file "$f" as alias' -e \\
-                'tell application "Finder" to set $attr of file fp to "$v"');
-        }
-        if (defined $cmd) {
-            $err = system $cmd . '>/dev/null 2>&1'; # (pipe all output to /dev/null)
+            push @cmd, '/usr/bin/osascript', '-e', qq(set fp to POSIX file "$f" as alias),
+                '-e', qq(tell application "Finder" to set $attr of file fp to "$v");
         }
+        $err = System(@cmd) if @cmd;
         if (not $err) {
             $et->VerboseValue("+ $tag", $val) if defined $val;
             $result = 1;
         } elsif (not $silentErr or $err != $silentErr) {
-            $cmd =~ s/ .*//s;
+            my $cmd = $cmd[0] || 'tag';
+            $cmd =~ s(.*/)();
             $et->Warn(qq{Error $err running "$cmd" to set $tag});
             $result = -1 unless $result;
         }
@@ -498,6 +508,7 @@ sub ExtractMDItemTags($$)
             $_ = '' if $_ eq '(null)';
             s/^"// and s/"$//;  # remove quotes if they exist
             s/\\"/"/g;          # un-escape quotes
+            s/\\\\/\\/g;        # un-escape backslashes
             $_ = $et->Decode($_, 'UTF8');
             push @$val, $_;
             next;
@@ -736,7 +747,7 @@ Writable tags use "xattr", "setfile" or "osascript" for writing.
 
 =head1 AUTHOR
 
-Copyright 2003-2025, Phil Harvey (philharvey66 at gmail.com)
+Copyright 2003-2026, Phil Harvey (philharvey66 at gmail.com)
 
 This library is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
